package org.artifactory.queue.services;

import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.ListUtils;
import org.artifactory.queue.QueueEventException;
import org.artifactory.storage.event.EventInfo;
import org.jfrog.common.ClockUtils;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * a Human readable report for a running queue
 * enable report by changing PersistentQueueRunnerReport's log level to DEBUG
 * @author avishayh
 */
@Slf4j
public class PersistentQueueRunnerReport {

    private static final ThreadLocal<PersistentQueueRunnerReport> tlReport = new ThreadLocal<>();
    private final Set<Long> filtered = new HashSet<>();
    private final List<EventInfo> loaded = new ArrayList<>();
    private final Map<String, Exception> exceptions = new LinkedHashMap<>();
    private final Set<Long> includedErrors = new HashSet<>();
    private String name;
    private long startTime;
    private long endTime;
    private long startMarker = -1;
    private long endMarker = -1;
    private PersistentQueue.TriggerResult triggerResult;

    public static void start(String name) {
        if (isEnabled()) {
            PersistentQueueRunnerReport report = new PersistentQueueRunnerReport();
            tlReport.set(report);
            report.startTime = ClockUtils.epochMillis();
            report.name = name;
        }
    }

    public static void end(PersistentQueue.TriggerResult triggerResult) {
        if (isStarted()) {
            final PersistentQueueRunnerReport report = report();
            report.triggerResult = triggerResult;
            report.endTime = ClockUtils.epochMillis();
            log.debug(report.toString());
            tlReport.remove();
        }
    }

    public static void addFilterEvents(List<EventInfo> before, List<EventInfo> after) {
        if (isStarted()) {
            //noinspection unchecked
            List<EventInfo> removed = ListUtils.subtract(before, after);
            report().filtered.addAll(removed.stream().map(EventInfo::getEventId).collect(Collectors.toSet()));
        }
    }

    public static void loadedEvents(List<EventInfo> events) {
        if (isStarted()) {
            report().loaded.addAll(events);
        }
    }

    public static void startMarker(long startMarker) {
        if (isStarted()) {
            report().startMarker = startMarker;
        }
    }

    public static void endMarker(long endMarker) {
        if (isStarted()) {
            report().endMarker = endMarker;
        }
    }

    public static void addException(String key, Exception exception) {
        if (isStarted()) {
            report().exceptions.put(key, exception);
        }
    }

    public static void includedErrors(Collection<Long> errors) {
        if (isStarted()) {
            report().includedErrors.addAll(errors);
        }
    }

    private static boolean isEnabled() {
        return log.isDebugEnabled();
    }

    private static boolean isStarted() {
        return report() != null;
    }

    private static PersistentQueueRunnerReport report() {
        return tlReport.get();
    }

    @Override
    public String toString() {
        long lapTime = endTime - startTime;
        int lapSecs = (int) (lapTime / 1000);
        //create human readable report
        StringBuilder sb = new StringBuilder();
        sb.append(String.format("\nname: %s\n", name));
        sb.append(String.format("lapTime: %d ms (%d secs)\n", lapTime, lapSecs));
        sb.append(String.format("status: %s (%d rows loaded, %d rows filtered)\n", triggerResult, loaded.size(),
                filtered.size()));
        sb.append(String.format("marker:   %s -> %s\n",
                startMarker == -1 ? "n/a" : startMarker,
                endMarker == -1 ? "n/a" : endMarker));
        if (!loaded.isEmpty()) {
            sb.append(String.format("loaded:   %s\n",
                    loaded.stream().map(EventInfo::getEventId).collect(Collectors.toSet())));
        }
        if (!filtered.isEmpty()) {
            sb.append(String.format("filtered: %s\n", filtered));
        }
        if (!exceptions.isEmpty()) {
            sb.append("failed: [\n");
            exceptions.forEach((key, exception) ->
                    sb.append(String.format("%s -> %s\n", key, exception))
            );
            sb.append("]\n");
        }
        if (!loaded.isEmpty()) {
            sb.append("events: [\n");
            Set<Long> failures =
                    exceptions.values().stream()
                            .filter(exception -> exception instanceof QueueEventException)
                            .map(exception -> ((QueueEventException) exception).getFailedEvent().getEventId())
                            .collect(Collectors.toSet());
            loaded.forEach(e -> {
                String error = includedErrors.contains(e.getEventId()) ? "|error" : "";
                String filter = filtered.contains(e.getEventId()) ? " -> filtered" : "";
                String failure = failures.contains(e.getEventId()) ? " -> failed" : "";
                sb.append(String.format("%d -> %s%s%s%s\n", e.getEventId(), e, error, filter, failure));
            });
            sb.append("]\n");
        }
        return sb.toString();
    }
}
