package org.artifactory.repo.interceptor;

import lombok.extern.slf4j.Slf4j;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.api.security.SecurityService;
import org.artifactory.common.MutableStatusHolder;
import org.artifactory.exception.CancelException;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.interceptor.storage.StorageInterceptorAdapter;
import org.artifactory.repo.system.SystemRepos;
import org.artifactory.sapi.fs.VfsItem;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import static org.apache.http.HttpStatus.SC_FORBIDDEN;
import static org.artifactory.repo.system.SystemRepos.USAGE_REPO_KEY;

/**
 * @author Tomer Slutsky
 */
@Slf4j
@Service
public class ObservabilityInterceptor extends StorageInterceptorAdapter {

    private final AuthorizationService authorizationService;

    @Autowired
    public ObservabilityInterceptor(AuthorizationService authorizationService) {
        this.authorizationService = authorizationService;
    }

    @Override
    public void beforeCreate(VfsItem fsItem, MutableStatusHolder statusHolder) {
        if (SystemRepos.isObservabilityRepo(fsItem.getRepoKey())) {
            if (!SecurityService.USER_SYSTEM.equals(authorizationService.currentUsername())) {
                setStatusError("Deployment into " + fsItem.getRepoKey() + " repo is forbidden", statusHolder);
            }
        }
    }

    @Override
    public void assertDeleteRepoAllowed(String repoKey, MutableStatusHolder status) {
        if (SystemRepos.isObservabilityRepo(repoKey)) {
            status.error("Deletion of Observability repository " + repoKey + " is not allowed.", SC_FORBIDDEN, log);
        }
    }

    @Override
    public void beforeDelete(VfsItem fsItem, MutableStatusHolder statusHolder, boolean moved) {
        if (!moved) {
            if (USAGE_REPO_KEY.equalsIgnoreCase(fsItem.getRepoKey()) &&
                    !SecurityService.USER_SYSTEM.equals(authorizationService.currentUsername())) {
                setStatusError("Deletion of artifacts from repository " + fsItem.getRepoKey() + " is not allowed.",
                        statusHolder);
            }
        }
    }

    @Override
    public boolean isCopyOrMoveAllowed(VfsItem sourceItem, RepoPath targetRepoPath, MutableStatusHolder status) {
        if (SystemRepos.isObservabilityRepo(sourceItem.getRepoKey())) {
            if (!SecurityService.USER_SYSTEM.equals(authorizationService.currentUsername())) {
                setStatusError("Copy/Move from " + sourceItem.getRepoKey() + " repo is forbidden", status);
                return false;
            }
        } else if (SystemRepos.isObservabilityRepo(targetRepoPath.getRepoKey())) {
            if (!SecurityService.USER_SYSTEM.equals(authorizationService.currentUsername())) {
                setStatusError("Copy/Move into " + targetRepoPath.getRepoKey() + " repo is forbidden", status);
                return false;
            }
        }
        return true;
    }

    private void setStatusError(String errMsg, MutableStatusHolder statusHolder) {
        statusHolder.error(errMsg, SC_FORBIDDEN, new CancelException(errMsg, SC_FORBIDDEN), log);
    }
}
