package org.artifactory.repo.interceptor;

import org.artifactory.api.common.BasicStatusHolder;
import org.artifactory.api.security.AuthorizationService;
import org.artifactory.api.security.SecurityService;
import org.artifactory.common.StatusEntry;
import org.artifactory.repo.RepoPath;
import org.artifactory.repo.RepoPathFactory;
import org.artifactory.sapi.fs.VfsItem;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.DataProvider;
import org.testng.annotations.Test;

import static org.artifactory.repo.system.SystemRepos.LOG_SHIPPING_REPO_KEY;
import static org.artifactory.repo.system.SystemRepos.USAGE_REPO_KEY;
import static org.mockito.Mockito.when;
import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertFalse;
import static org.testng.Assert.assertTrue;

/**
 * @author Tomer Slutsky
 */
public class ObservabilityInterceptorTest {
    @Mock
    private AuthorizationService authorizationService;
    @Mock
    private VfsItem item;
    private ObservabilityInterceptor observabilityInterceptor;
    private AutoCloseable mocks;

    @DataProvider
    public static Object[][] deleteProvider() {
        return new Object[][] {
                {LOG_SHIPPING_REPO_KEY, false},
                {LOG_SHIPPING_REPO_KEY, true},
                {USAGE_REPO_KEY, true},
                {USAGE_REPO_KEY, false}
        };
    }

    @DataProvider
    public static Object[][] beforeCreateProviderFail() {
        return new Object[][] {
                {LOG_SHIPPING_REPO_KEY, false},
                {USAGE_REPO_KEY, false},
        };
    }

    @DataProvider
    public static Object[][] beforeCreateProviderSuccess() {
        return new Object[][] {
                {LOG_SHIPPING_REPO_KEY, true},
                {USAGE_REPO_KEY, true},
                {"random", true},
                {"random", false},
        };
    }

    @DataProvider
    public static Object[][] beforeDeleteProviderSuccess() {
        return new Object[][] {
                {LOG_SHIPPING_REPO_KEY},
                {"random"}
        };
    }

    @DataProvider
    public static Object[][] copyMoveProviderFailFrom() {
        return new Object[][] {
                {LOG_SHIPPING_REPO_KEY, "random", false},
                {USAGE_REPO_KEY, "random", false},
                {USAGE_REPO_KEY, LOG_SHIPPING_REPO_KEY, false},
                {LOG_SHIPPING_REPO_KEY, LOG_SHIPPING_REPO_KEY, false}
        };
    }

    @DataProvider
    public static Object[][] copyMoveProviderFailTo() {
        return new Object[][] {
                {"random", LOG_SHIPPING_REPO_KEY, false},
                {"random", USAGE_REPO_KEY, false}
        };
    }

    @DataProvider
    public static Object[][] copyMoveProviderAllowed() {
        return new Object[][] {
                {"random", LOG_SHIPPING_REPO_KEY, true},
                {"random", USAGE_REPO_KEY, true},
                {LOG_SHIPPING_REPO_KEY, LOG_SHIPPING_REPO_KEY, true},
                {USAGE_REPO_KEY, USAGE_REPO_KEY, true}
        };
    }

    @BeforeMethod
    public void setUp() {
        mocks = MockitoAnnotations.openMocks(this);
        observabilityInterceptor = new ObservabilityInterceptor(authorizationService);
    }

    @AfterMethod
    public void tearDown() throws Exception {
        mocks.close();
    }

    @Test(dataProvider = "deleteProvider")
    public void testAssertDeleteRepoNotAllowed(String repoKey, boolean isSystem) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockAuthService(isSystem);
        observabilityInterceptor.assertDeleteRepoAllowed(repoKey, statusHolder);
        assertStatusHolder(statusHolder, "Deletion of Observability repository " + repoKey + " is not allowed.");
    }

    @Test
    public void testAssertDeleteRepoPassesOnRegularRepo() {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockAuthService(false);
        observabilityInterceptor.assertDeleteRepoAllowed("random", statusHolder);
        assertFalse(statusHolder.isError());
    }

    @Test(dataProvider = "beforeCreateProviderFail")
    public void testBeforeCreateFails(String repoKey, boolean isSystem) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockAuthService(isSystem);
        mockItem(repoKey);
        observabilityInterceptor.beforeCreate(item, statusHolder);
        assertStatusHolder(statusHolder, "Deployment into " + repoKey + " repo is forbidden");

    }

    @Test(dataProvider = "beforeCreateProviderSuccess")
    public void testBeforeCreatePasses(String repoKey, boolean isSystem) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockAuthService(isSystem);
        mockItem(repoKey);
        observabilityInterceptor.beforeCreate(item, statusHolder);
        assertFalse(statusHolder.isError());

    }

    @Test(dataProvider = "beforeDeleteProviderSuccess")
    public void testBeforeDeletePasses(String repoKey) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockItem(repoKey);
        observabilityInterceptor.beforeDelete(item, statusHolder, false);
        assertFalse(statusHolder.isError());

    }

    @Test
    public void testBeforeDeleteFails() {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockItem(USAGE_REPO_KEY);
        observabilityInterceptor.beforeDelete(item, statusHolder, false);
        assertStatusHolder(statusHolder,
                "Deletion of artifacts from repository " + USAGE_REPO_KEY + " is not allowed.");

    }

    @Test
    public void testBeforeDeletePassesUsage() {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockItem(USAGE_REPO_KEY);
        mockAuthService(true);
        observabilityInterceptor.beforeDelete(item, statusHolder, false);
        assertFalse(statusHolder.isError());
    }

    @Test(dataProvider = "copyMoveProviderFailFrom")
    public void testIsCopyOrMoveAllowedFailsFrom(String repoKeyFrom, String repoKeyTo, boolean isSystem) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockAuthService(isSystem);
        mockItem(repoKeyFrom);
        RepoPath repoPath = RepoPathFactory.create(repoKeyTo, "");
        assertFalse(observabilityInterceptor.isCopyOrMoveAllowed(item, repoPath, statusHolder));
        assertStatusHolder(statusHolder, "Copy/Move from " + repoKeyFrom + " repo is forbidden");
    }

    @Test(dataProvider = "copyMoveProviderFailTo")
    public void testIsCopyOrMoveAllowedFailsTo(String repoKeyFrom, String repoKeyTo, boolean isSystem) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockAuthService(isSystem);
        mockItem(repoKeyFrom);
        RepoPath repoPath = RepoPathFactory.create(repoKeyTo, "");
        assertFalse(observabilityInterceptor.isCopyOrMoveAllowed(item, repoPath, statusHolder));
        assertStatusHolder(statusHolder, "Copy/Move into " + repoKeyTo + " repo is forbidden");
    }

    @Test(dataProvider = "copyMoveProviderAllowed")
    public void testIsCopyOrMoveAllowed(String repoKeyFrom, String repoKeyTo, boolean isSystem) {
        BasicStatusHolder statusHolder = new BasicStatusHolder();
        mockAuthService(isSystem);
        mockItem(repoKeyFrom);
        RepoPath repoPath = RepoPathFactory.create(repoKeyTo, "");
        assertTrue(observabilityInterceptor.isCopyOrMoveAllowed(item, repoPath, statusHolder));
        assertFalse(statusHolder.isError());
    }

    private void assertStatusHolder(BasicStatusHolder statusHolder, String errorMessage) {
        assertTrue(statusHolder.isError());
        assertEquals(statusHolder.getErrors().size(), 1);
        StatusEntry lastError = statusHolder.getLastError();
        assertEquals(lastError.getStatusCode(), 403);
        assertEquals(lastError.getMessage(), errorMessage);
    }

    private void mockItem(String repoKey) {
        when(item.getRepoKey()).thenReturn(repoKey);
    }

    private void mockAuthService(boolean isSystem) {
        when(authorizationService.currentUsername())
                .thenReturn(isSystem ? SecurityService.USER_SYSTEM : "notsystem");
    }
}