package org.artifactory.storage.db.event.service;

import org.artifactory.storage.StorageException;
import org.artifactory.storage.db.event.dao.EventsDao;
import org.artifactory.storage.db.event.dao.NodeEventsCursorDao;
import org.artifactory.storage.db.event.entity.DbNodeEventCursor;
import org.artifactory.storage.db.event.entity.NodeEventCursorType;
import org.artifactory.storage.db.event.model.NodeEventCursor;
import org.artifactory.storage.db.event.service.mapper.NodeEventCursorMapper;
import org.artifactory.storage.db.event.service.mapper.NodeEventCursorMapperImpl;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.sql.SQLException;
import java.util.List;
import java.util.Optional;

/**
 * @author Uriah Levy
 */
@Service
public class NodeEventCursorServiceImpl implements NodeEventCursorService {
    private static final Logger log = LoggerFactory.getLogger(NodeEventCursorServiceImpl.class);

    private final NodeEventsCursorDao nodeEventsCursorDao;
    private final EventsDao eventsDao;
    private final NodeEventCursorMapper nodeEventCursorMapper;

    @Autowired
    NodeEventCursorServiceImpl(NodeEventsCursorDao nodeEventsCursorDao, EventsDao eventsDao) {
        this.nodeEventsCursorDao = nodeEventsCursorDao;
        this.eventsDao = eventsDao;
        this.nodeEventCursorMapper = new NodeEventCursorMapperImpl();
    }

    @Override
    public void updateOrInsertCursor(NodeEventCursor eventCursor) {
        try {
            log.debug("Updating cursor for operator '{}' with timestamp {}", eventCursor.getOperatorId(),
                    eventCursor.getEventMarker());
            DbNodeEventCursor dbCursor = nodeEventCursorMapper.nodeEventCursorToDbNodeEventCursor(eventCursor);
            int updateResult = nodeEventsCursorDao.update(dbCursor);
            if (updateResult == 0) {
                nodeEventsCursorDao.insert(dbCursor);
            }
        } catch (SQLException e) {
            throw new StorageException("Could not update cursor for operator with ID '" + eventCursor.getOperatorId() +
                    "'", e);
        }
    }

    @Override
    public void insertCursor(String operatorId, long marker, NodeEventCursorType nodeEventCursorType) {
        try {
            nodeEventsCursorDao.insert(nodeEventCursorMapper
                    .nodeEventCursorToDbNodeEventCursor(new NodeEventCursor(operatorId, marker, nodeEventCursorType)));
        } catch (SQLException e) {
            throw new StorageException("Could not insert cursor for operator with ID '" + operatorId + "'", e);
        }
    }

    @Override
    public NodeEventCursor cursorForOperator(String operatorId) {
        try {
            Optional<DbNodeEventCursor> dbNodeEventCursorOptional = nodeEventsCursorDao.cursorForOperator(operatorId);
            return dbNodeEventCursorOptional
                    .map(value -> nodeEventCursorMapper.dbNodeEventCursorToNodeEventCursor(value))
                    .orElse(null);
        } catch (SQLException e) {
            throw new StorageException("Unable to retrieve cursor for operator with ID " + operatorId, e);
        }
    }

    @Override
    public void deleteCursor(String operationId) {
        try {
            nodeEventsCursorDao.delete(operationId);
        } catch (SQLException e) {
            throw new StorageException("Unable to delete cursor with ID " + operationId, e);
        }
    }

    @Override
    public long oldestTimestampForEventLogOperator() {
        try {
            //cursor table holds marker which represent either timestamp or eventId
            //for timestamp types we just query the minimum timestamp from the cursor table
            //for event_id types we need to query the minimum event_id from the cursor table and than the timestamp
            //from node_events table
            //returning the minimum timestamp between the 2 of them.
            List<NodeEventCursorType> eventIds = NodeEventCursorType.getEventIdLogTypes();
            List<NodeEventCursorType> timestamps = NodeEventCursorType.getTimestampLogTypes();
            long oldestTimestamp = nodeEventsCursorDao.oldestMarkerForTypes(timestamps);
            long oldestEventId = nodeEventsCursorDao.oldestMarkerForTypes(eventIds);
            long oldestTimestampByEventId = eventsDao.getOldestTimestampByEventIdInclusive(oldestEventId);

            // protection if no by timestamp cursors
            if (oldestTimestamp > 0 && oldestTimestampByEventId > 0) {
                return Math.min(oldestTimestamp, oldestTimestampByEventId);
            } else if (oldestTimestamp > 0) {
                return oldestTimestamp;
            } else {
                return oldestTimestampByEventId;
            }
        } catch (SQLException e) {
            throw new StorageException("Error when trying to get oldest timestamp from event log", e);
        }
    }

    @Override
    public void updateCursor(String operatorId, long marker) {
        try {
            nodeEventsCursorDao.update(operatorId, marker);
        } catch (SQLException e) {
            throw new StorageException("Could not update cursor for operator with ID: " + operatorId, e);
        }
    }
}