/*
 *
 * Artifactory is a binaries repository manager.
 * Copyright (C) 2018 JFrog Ltd.
 *
 * Artifactory is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 *
 * Artifactory is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with Artifactory.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package org.artifactory.storage.db.version;

import org.artifactory.storage.db.conversion.version.v213.V213ConversionFailFunction;
import org.artifactory.storage.db.conversion.version.v213.V213ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v215.V215ConversionFailFunction;
import org.artifactory.storage.db.conversion.version.v215.V215ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v220.V220ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v221.V221ConversionPredicateNodePropsIndex;
import org.artifactory.storage.db.conversion.version.v222.V222ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v224.V224ConversionPredicates;
import org.artifactory.storage.db.conversion.version.v224.V224OptionalDeleteTableDBConverter;
import org.artifactory.storage.db.conversion.version.v225.V225ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v225.V225ConversionPredicateBundleFiles;
import org.artifactory.storage.db.conversion.version.v225.V225LockingDBSqlConverter;
import org.artifactory.storage.db.conversion.version.v226.V226ConversionPredicateBuildsRepoColumn;
import org.artifactory.storage.db.conversion.version.v227.V227ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v227.V227CreateDbSpecificSequence;
import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsAddColumnConversionPredicate;
import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsErrorAddColumnConversionPredicate;
import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsErrorAddConstraintConversionPredicate;
import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsErrorCreateIndexConversionPredicate;
import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsErrorDropIndexConversionPredicate;
import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsTmpIndexConversionPredicate;
import org.artifactory.storage.db.conversion.version.v227.V227NodeEventsTmpTableConversionPredicate;
import org.artifactory.storage.db.conversion.version.v229.V229ConversionPredicateNodeEventsTempIndex;
import org.artifactory.storage.db.conversion.version.v230.V230ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v232.V232ConversionPredicate;
import org.artifactory.storage.db.conversion.version.v233.V233ConversionPredicate;
import org.artifactory.storage.db.version.converter.AsyncDBSqlConditionalConverter;
import org.artifactory.storage.db.version.converter.ConditionalDBSqlConverter;
import org.artifactory.storage.db.version.converter.DBConverter;
import org.artifactory.storage.db.version.converter.DBSqlConverter;
import org.artifactory.storage.db.version.converter.OptionalDBConverter;
import org.artifactory.version.ArtifactoryVersion;
import org.artifactory.version.ArtifactoryVersionProvider;

/**
 * NOTE! The conversion logic for this enum is different from the one used by
 * {@link org.artifactory.version.ArtifactoryConfigVersion}: here the converters are *in line* with the version that
 * requires them.
 * Meaning for example, that passing into 4.4.1 (denoted v441) requires that you run conversion for db version 'v441'
 * and this db version is the one used until 4.14.0 (denoted v4140)
 * Artifactory DB version
 *
 * @author Gidi Shabat
 */
public enum ArtifactoryDBVersion {
    v100(ArtifactoryVersionProvider.v300.get()),
    v209(ArtifactoryVersionProvider.v600m023.get(), new DBSqlConverter("v209_bundle_files")),
    v210(ArtifactoryVersionProvider.v610m002.get(), new DBSqlConverter("v210_trusted_keys_mandatory")),
    v211(ArtifactoryVersionProvider.v620.get(), new DBSqlConverter("v211_server_id")),
    v212(ArtifactoryVersionProvider.v640m007.get(), new DBSqlConverter("v212_rb_repo")),
    v213(ArtifactoryVersionProvider.v650m001.get(),
            new OptionalDBConverter("v213_node_props_index", new V213ConversionPredicate(),
                    new V213ConversionFailFunction())),
    v214(ArtifactoryVersionProvider.v660m001.get(), new DBSqlConverter("v214_replication_errors"),
            new DBSqlConverter("v214_remove_unique_index")),
    v215(ArtifactoryVersionProvider.v682m001.get(),
            new OptionalDBConverter("v215_original_content_type", new V215ConversionPredicate(),
                    new V215ConversionFailFunction())),
    v216(ArtifactoryVersionProvider.v690m001.get(), new DBSqlConverter("v216_cursor_priorities_migrations")),
    v217(ArtifactoryVersionProvider.v6110m001.get(), new DBSqlConverter("v217_create_jobs_table"),
            new DBSqlConverter("v217_set_default_storing_repo"),
            new DBSqlConverter("v217_update_artifact_bundles_indexes"),
            new DBSqlConverter("v217_remove_metadata_migration_status")),
    v218(ArtifactoryVersionProvider.v6120m079.get(), new DBSqlConverter("v218_add_bundle_files_idx")),
    v219(ArtifactoryVersionProvider.v6170m001.get(), new DBSqlConverter("v219_add_type_to_node_event_cursor")),
    v220(ArtifactoryVersionProvider.v732m017.get(),
            new ConditionalDBSqlConverter("v220_change_tasks_context_size", new V220ConversionPredicate())),
    v221(ArtifactoryVersionProvider.v740m081.get(), new ConditionalDBSqlConverter("v221_update_node_props_index",
            new V221ConversionPredicateNodePropsIndex())),
    v222(ArtifactoryVersionProvider.v750m145.get(),
            new ConditionalDBSqlConverter("v222_change_bundle_version_length", new V222ConversionPredicate())),
    v223(ArtifactoryVersionProvider.v760m001.get(), new DBSqlConverter("v223_change_artifactory_version_length")),
    v224(ArtifactoryVersionProvider.v770.get(),
            new ConditionalDBSqlConverter("v224_change_replication_errors_to_node_event_errors",
                    new V224ConversionPredicates.V224ConversionPredicate()),
            new DBSqlConverter("v224_change_server_role_length"),
            new V224OptionalDeleteTableDBConverter()),
    v225(ArtifactoryVersionProvider.v7110m033.get(),
            new AsyncDBSqlConditionalConverter("v225_change_nodes_node_name_idx",
                    new V225ConversionPredicate("nodes_node_name_idx", "node_name varchar_pattern_ops")),
            new AsyncDBSqlConditionalConverter("v225_change_nodes_node_path_idx",
                    new V225ConversionPredicate("nodes_node_path_idx", "node_path varchar_pattern_ops")),
            new AsyncDBSqlConditionalConverter("v225_change_nodes_node_repo_path_idx",
                    new V225ConversionPredicate("nodes_repo_path_name_idx", "repo varchar_pattern_ops",
                            "node_path varchar_pattern_ops", "node_name varchar_pattern_ops")),
            new AsyncDBSqlConditionalConverter("v225_add_bundle_files_node_id_index",
                    new V225ConversionPredicateBundleFiles()),
            new V225LockingDBSqlConverter("v225_alter_distributed_locks_set_unlogged")),
    v226(ArtifactoryVersionProvider.v7130m001.get(), new ConditionalDBSqlConverter("v226_add_repo_column_to_builds",
            new V226ConversionPredicateBuildsRepoColumn())),
    v227(ArtifactoryVersionProvider.v7170m040.get(),
            new V227CreateDbSpecificSequence("v227"),
            new ConditionalDBSqlConverter("v227_node_events_tmp_table",
                    new V227NodeEventsTmpTableConversionPredicate()),
            new ConditionalDBSqlConverter("v227_node_events_tmp_index",
                    new V227NodeEventsTmpIndexConversionPredicate()),
            new ConditionalDBSqlConverter("v227_node_events_add_column",
                    new V227NodeEventsAddColumnConversionPredicate()),
            new ConditionalDBSqlConverter("v227_node_events_errors_add_column",
                    new V227NodeEventsErrorAddColumnConversionPredicate()),
            new ConditionalDBSqlConverter("v227_node_events_errors_add_constraint",
                    new V227NodeEventsErrorAddConstraintConversionPredicate()),
            new ConditionalDBSqlConverter("v227_node_events_errors_drop_index",
                    new V227NodeEventsErrorDropIndexConversionPredicate()),
            new ConditionalDBSqlConverter("v227_node_events_errors_create_index",
                    new V227NodeEventsErrorCreateIndexConversionPredicate()),

            new ConditionalDBSqlConverter("v228_node_meta_infos", new V227ConversionPredicate())
    ),
    v229(ArtifactoryVersionProvider.v7201m001.get(),
            new AsyncDBSqlConditionalConverter("v229_node_events_tmp_event_id_idx",
                    new V229ConversionPredicateNodeEventsTempIndex())),
    v230(ArtifactoryVersionProvider.v7201m001.get(), new ConditionalDBSqlConverter("v230_fix_v224_if_needed",
            new V230ConversionPredicate())),
    v231(ArtifactoryVersionProvider.v7230m012.get()),
    v232(ArtifactoryVersionProvider.v7250m001.get(),
            new ConditionalDBSqlConverter("v232_change_watches_username_length", new V232ConversionPredicate())),
    v233(ArtifactoryVersionProvider.v7252m001.get(), new ConditionalDBSqlConverter("v233_add_build_module_type",
            new V233ConversionPredicate())),
    ;

    private final DBConverter[] converters;
    private final ArtifactoryVersion version;

    ArtifactoryDBVersion(ArtifactoryVersion version, DBConverter... converters) {
        this.version = version;
        this.converters = converters;
    }

    public ArtifactoryVersion getVersion() {
        return version;
    }

    public DBConverter[] getConverters() {
        return converters;
    }
}
